# startup script

import sys

# add /lib for additional python modules
sys.path.append('./lib')

import ConfigParser
import os

# do not use CollectModules. it is used to collect the list of used libraries so
# that the user doesnt have to install the whole python library.
def CollectModules():
    import shutil
    print "Collecting Modules..."
    localdir = os.listdir(".")
    pth = os.path.dirname(__file__)
    ldir = os.listdir(pth)
    targetdir = pth + os.sep + 'lib'
    for i in sys.modules.items():    
        try:    
            s = i[1].__file__
        except:
            print "- skipping " + str(i[0]) + " (is built-in)"
            s = None
        if s:
            if (os.path.splitext(s)[1] == '.pyc'):
                as = os.path.splitext(s)[0] + '.py'
                print "trying alternate file '" + as + "'"
                if os.path.exists(as):
                    s = as
            bn = os.path.basename(s)
            if (bn not in ldir) and (bn not in localdir): 
                v = i[0].split('.')
                print "copying " + str(i[0]) + "..."
                if len(v) > 1:
                    shutil.copy(s,targetdir + os.sep + v[0])
                else:
                    shutil.copy(s,targetdir)
            else:
                print "*** skipping " + str(i[0]) + " (is local)"

# CollectModules()

# load exported nc functions
from qc import *

# loads all plugins (and recursive) from path
def LoadPlugins(path):
    d = os.listdir(path)
    d.sort()
    for i in d:
        if os.path.isdir(i): # recurse subdirectories
            LoadPlugins(path + os.sep + i)
        elif os.path.splitext(i)[1] == '.qcplugin':
            UsePlugin(path + os.sep + i)

def GetObjectInfoString(object):
    return "%s %s" % (GetObjectClassName(object),GetObjectName(object))

def GetModuleclassMap():
    cm = {}
    for i in GetModuleclassObjects():
        cm[GetObjectName(i)] = i
    return cm

def GetWindowclassMap():
    wcm = {}
    for i in GetWindowclassObjects():
        wcm[GetObjectName(i)] = i
    return wcm

def GetClassMap():
    cm = {}
    for i in GetModuleclassObjects():
        cm[GetObjectName(i)] = i
    for i in GetWindowclassObjects():
        cm[GetObjectName(i)] = i
    return cm

def GetLinkMap(tf):
    lm = {}
    for i in GetLinkObjects(tf):
        lm[GetObjectName(i)] = (i,GetLinkObjectType(i),GetLinkObjectDataName(i))
    return lm

def GetModuleclassObject(name):
    for i in GetModuleclassObjects():
        if GetObjectName(i) == name:
            return i    

def GetWindowclassObject(name):
    for i in GetWindowclassObjects():
        if GetObjectName(i) == name:
            return i

def GetLinkObject(tf,name):
    for i in GetLinkObjects(tf):
        if GetObjectName(i) == name:
            return i

def GetDataInfoObject(name):
    for i in GetDataInfoObjects():
        if GetObjectName(i) == name:
            return i

def GetDataInfoMap():
    dim = {}
    for i in GetDataInfoObjects():
        dim[GetObjectName(i)] = i
    return dim

def LoadMacros(path):
    ldir = os.listdir(path)
    for i in ldir:
        if os.path.splitext(i)[1] == '.py':
            exec('import ' + os.path.splitext(i)[0])

def AddTemplates(o,path):
    ldir = os.listdir(path)
    for i in ldir:
        if os.path.isdir(path + os.sep + i):
            fobj = CreateFolderObject(o)
            RenameObject(fobj,i)
            AddTemplates(fobj,path + os.sep + i)
        elif os.path.splitext(i)[1] == '.qcprj':
            flink = CreateFilelinkObject(path + os.sep + i)
            SetParentObject(o,flink)

def AddClasses(o,pathfile,classmap):    
    s = pathfile.readline()
    while s:        
        s = s.replace("/r","")
        s = s.replace("/n","")
        s = s.expandtabs(1)
        s = s.strip()
        if s.startswith('/') and (len(s) > 1): # create a folder or go back one step
            fname = s[1:]
            if fname != '..':
                fo = CreateFolderObject(o)
                RenameObject(fo,fname)
                AddClasses(fo,pathfile,classmap)
            else:
                return
        elif s:
            if classmap.has_key(s):
                SetParentObject(o,classmap[s])
        s = pathfile.readline()

def AddVstClasses(ovst,ovsti,classmap):
    for i in classmap.values():
        if (not GetObjectParent(i)):
            n = GetObjectName(i)
            if (n[len(n)-5:] == "(VST)"):
                SetParentObject(ovst,i)
            elif (n[len(n)-6:] == "(VSTi)"):
                SetParentObject(ovsti,i)

def AddClassesThatHaveNoParent(o,classmap):
    for i in classmap.values():
        if not GetObjectParent(i):
            SetParentObject(o,i)

def LoadColors(colorfile):
    cp = ConfigParser.ConfigParser()
    cp.read([colorfile])
    if(cp.has_section("DataInfoColors")):
        dim = GetDataInfoMap()
        for i in cp.options("DataInfoColors"):
            v = eval(cp.get("DataInfoColors",i))
            print v[1:]
            if(dim.has_key(v[0])):
                SetDataInfoObjectColor(dim[v[0]],v[1:])

# some useless stats to impress people
def PrintPoserStats():
    print "Total: %i Module Classes, %i Window Classes." % (len(GetModuleclassObjects()),len(GetWindowclassObjects()))

# # # startup # # # 

# add /macros for additional python macros
sys.path.append(os.path.dirname(__file__) + os.sep + "macros")

# init
LoadPlugins(os.path.dirname(__file__) + os.sep + "plugins")
LoadMacros(os.path.dirname(__file__) + os.sep + "macros")

# setup colors
LoadColors(os.path.dirname(__file__) + os.sep + "colors.ini")

def OnStartup():    
    PrintPoserStats()
    repos_obj = GetRepositoryObject()
    try:
        f = open(os.path.dirname(__file__) + os.sep + "repository.idx","r")
        AddClasses(repos_obj,f,GetClassMap())
        f.close()
    except:
        print "Couldnt open index file."
        
    # add vst objects    
    VstFolder = CreateFolderObject(repos_obj)
    RenameObject(VstFolder,"Virtual Studio Technology")
    EffectFolder = CreateFolderObject(VstFolder)
    RenameObject(EffectFolder,"Effects")
    InstrumentFolder = CreateFolderObject(VstFolder)
    RenameObject(InstrumentFolder,"Instruments")
    AddVstClasses(EffectFolder,InstrumentFolder,GetClassMap())
    
    # add remaining objects
    AddClassesThatHaveNoParent(repos_obj,GetClassMap())
    
    # add templates
    PatchFolder = CreateFolderObject(repos_obj)
    RenameObject(PatchFolder,"Templates")
    AddTemplates(PatchFolder,os.path.dirname(__file__) + os.sep + "templates")

    # create empty construct
    prj_obj = GetProjectObject()
    co = CreateConstructObject(prj_obj)
    RenameObject(co,"New Construct")
    OpenView(co)
    print ""
    print "Ready to rock."
    
def OnShutdown():
    print "OnShutdown"

def OnNewProject():
    return

SetHostEventMethod("OnStartup",OnStartup)
SetHostEventMethod("OnShutdown",OnShutdown)
SetHostEventMethod("OnNewProject",OnNewProject)


